<?php

/**
 * @file
 * Contains \Drupal\panels\Form\LayoutChangeSettings.
 */

namespace Drupal\panels\Form;

use Drupal\Component\Plugin\ConfigurablePluginInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\layout_plugin\Layout;
use Drupal\layout_plugin\Plugin\Layout\LayoutPluginManagerInterface;
use Drupal\user\SharedTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

class LayoutChangeSettings extends FormBase {

  /**
   * The layout plugin manager.
   *
   * @var \Drupal\layout_plugin\Plugin\Layout\LayoutPluginManagerInterface
   */
  protected $manager;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\user\SharedTempStoreFactory
   */
  protected $tempstore;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.layout_plugin'),
      $container->get('user.shared_tempstore')
    );
  }

  /**
   * LayoutChangeSettings constructor.
   *
   * @param \Drupal\layout_plugin\Plugin\Layout\LayoutPluginManagerInterface $manager
   *   The layout plugin manager.
   * @param \Drupal\user\SharedTempStoreFactory $tempstore
   *   The tempstore factory.
   */
  public function __construct(LayoutPluginManagerInterface $manager, SharedTempStoreFactory $tempstore) {
    $this->manager = $manager;
    $this->tempstore = $tempstore;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'panels_layout_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $cached_values = $form_state->getTemporaryValue('wizard');

    /* @var $variant_plugin \Drupal\panels\Plugin\DisplayVariant\PanelsDisplayVariant */
    $variant_plugin = $cached_values['plugin'];

    $form['old_layout'] = [
      '#title' => $this->t('Old Layout'),
      '#type' => 'select',
      '#options' => Layout::getLayoutOptions(['group_by_category' => TRUE]),
      '#default_value' => $cached_values['layout_change']['old_layout'],
      '#disabled' => TRUE,
      '#access' => !empty($cached_values['layout_change']),
    ];

    $form['new_layout'] = [
      '#title' => $this->t('New Layout'),
      '#type' => 'select',
      '#options' => Layout::getLayoutOptions(['group_by_category' => TRUE]),
      '#default_value' => $cached_values['layout_change']['new_layout'],
      '#disabled' => TRUE,
      '#access' => !empty($cached_values['layout_change']),
    ];

    // If a layout is already selected, show the layout settings.
    $form['layout_settings_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Layout settings'),
      '#tree' => TRUE,
    ];

    $layout_settings = !empty($cached_values['layout_change']['layout_settings']) ? $cached_values['layout_change']['layout_settings'] : [];
    if (!$layout_settings && $variant_plugin->getLayout() instanceof ConfigurablePluginInterface) {
      $layout_settings = $variant_plugin->getLayout()->getConfiguration();
    }
    $layout_id = !empty($cached_values['layout_change']['new_layout']) ? $cached_values['layout_change']['new_layout'] : $variant_plugin->getConfiguration()['layout'];
    $layout = Layout::layoutPluginManager()->createInstance($layout_id, $layout_settings);
    $form['layout_settings_wrapper']['layout_settings'] = $layout->buildConfigurationForm([], $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $cached_values = $form_state->getTemporaryValue('wizard');
    /** @var \Drupal\ctools\Wizard\EntityFormWizardInterface $wizard */
    $wizard = $form_state->getFormObject();
    $next_params = $wizard->getNextParameters($cached_values);
    /* @var $plugin \Drupal\panels\Plugin\DisplayVariant\PanelsDisplayVariant */
    $plugin = $cached_values['plugin'];
    $layout_id = !empty($cached_values['layout_change']['new_layout']) ? $cached_values['layout_change']['new_layout'] : $plugin->getConfiguration()['layout'];
    $layout = Layout::layoutPluginManager()->createInstance($layout_id, []);
    if ($layout instanceof PluginFormInterface) {
      $sub_form_state = new FormState();
      $plugin_values = $form_state->getValue(['layout_settings_wrapper', 'layout_settings']);
      if ($plugin_values) {
        $sub_form_state->setValues($plugin_values);
        $layout->submitConfigurationForm($form, $sub_form_state);
        if ($layout instanceof ConfigurablePluginInterface) {
          if (substr($next_params['step'], 0 -7) == 'regions') {
            $cached_values['layout_change']['layout_settings'] = $layout->getConfiguration();
          }
          else {
            $plugin->setLayout($layout, $layout->getConfiguration());
            $cached_values['plugin'] = $plugin;
            unset($cached_values['layout_change']);
          }
        }
      }
      else {
        if (substr($next_params['step'], 0 -7) == 'regions') {
          $cached_values['layout_change']['layout_settings'] = [];
        }
        else {
          $plugin->setLayout($layout, []);
          $cached_values['plugin'] = $plugin;
          unset($cached_values['layout_change']);
        }
      }
    }
    else {
      if (substr($next_params['step'], 0 -7) == 'regions') {
        $cached_values['layout_change']['layout_settings'] = [];
        }
      else {
        $plugin->setLayout($layout, []);
        $cached_values['plugin'] = $plugin;
        unset($cached_values['layout_change']);
      }
    }
    $form_state->setTemporaryValue('wizard', $cached_values);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $cached_values = $form_state->getTemporaryValue('wizard');
    /* @var $plugin \Drupal\panels\Plugin\DisplayVariant\PanelsDisplayVariant */
    $plugin = $cached_values['plugin'];
    $layout_id = !empty($cached_values['layout_change']['new_layout']) ? $cached_values['layout_change']['new_layout'] : $plugin->getConfiguration()['layout'];
    $layout = Layout::layoutPluginManager()->createInstance($layout_id, []);
    if ($layout instanceof PluginFormInterface) {
      $sub_form_state = new FormState();
      $plugin_values = $form_state->getValue(['layout_settings_wrapper', 'layout_settings']);
      if ($plugin_values) {
        $sub_form_state->setValues($plugin_values);
        $layout->validateConfigurationForm($form, $sub_form_state);
      }
    }
  }

}
